@@ -10,7 +10,7 @@
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -993,11 +993,12 @@
     {
       return;
     }
-  if (connectDestination == nil || connectSource == nil)
+  if (connectSource == nil)
     {
       return;
     }
-  if ([[self activeDocument] containsObject: connectDestination] == NO)
+  if (connectDestination
+      && [[self activeDocument] containsObject: connectDestination] == NO)
     {
       NSLog(@"Oops - connectDestination not in active document");
       return;
