@@ -15,7 +15,7 @@
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -52,6 +52,7 @@
 #include "GormObjectEditor.h"
 #include "GormWrapperBuilder.h"
 #include "GormWrapperLoader.h"
+#include "GormDocumentWindow.h"
 
 @interface GormDisplayCell : NSButtonCell
 @end
@@ -332,8 +333,11 @@ static NSImage  *fileImage = nil;
   id                    o = nil;
 
   // get the window and cache it...
-  window = [self _docWindow];
-
+  window = (GormDocumentWindow *)[self _docWindow];
+  [IBResourceManager registerForAllPboardTypes:window
+	  			inDocument:self];
+  [window setDocument: self];
+  
   // set up the toolbar...
   toolbar = [(NSToolbar *)[NSToolbar alloc] initWithIdentifier: @"GormToolbar"];
   [toolbar setAllowsUserCustomization: NO];
@@ -413,7 +417,7 @@ static NSImage  *fileImage = nil;
   /* classes view */
   mainRect.origin = NSMakePoint(0,0);
   classesView = [(GormClassEditor *)[GormClassEditor alloc] initWithDocument: self];
-  [classesView setFrame: mainRect];
+  // [classesView setFrame: mainRect];
   
   /*
    * Set the objects view as the initial view the user's see on startup.
@@ -504,6 +508,9 @@ static NSImage  *fileImage = nil;
     {
       [objectsView addObject: o];
     }
+
+  // set the file type in the prefs manager...
+  [filePrefsManager setFileTypeName: [self fileType]];  
 }
 
 /**
@@ -879,7 +886,8 @@ static NSImage  *fileImage = nil;
       {
 	[selectionBox setContentView: scrollView];
 	[toolbar setSelectedItemIdentifier: @"ObjectsItem"];
-	[self setSelectionFromEditor: objectsView];
+	if (![NSApp isConnecting])
+	  [self setSelectionFromEditor: objectsView];
       }
       break;
     case 1: // images
@@ -921,6 +929,25 @@ static NSImage  *fileImage = nil;
     }
 }
 
+- (NSView *) viewWithTag:(int)tag
+{
+  switch (tag)
+    {
+      case 0: // objects
+	return objectsView;
+      case 1: // images
+	return imagesView;
+      case 2: // sounds
+	return soundsView;
+      case 3: // classes
+	return classesView;
+      case 4: // file prefs
+        return filePrefsView;
+      default: 
+        return nil;
+    }
+}
+
 - (void) changeToTopLevelEditorAcceptingTypes: (NSArray *)types
 				  andFileType: (NSString *)fileType
 {
@@ -1125,15 +1152,6 @@ static NSImage  *fileImage = nil;
   return [aPasteboard setData: data forType: aType];
 }
 
-/**
- * Create a subclass of the currently selected class in the classes view.
- */
-- (id) createSubclass: (id)sender
-{
-  [classesView createSubclass];
-  return self;
-}
-
 /**
  * The given pasteboard chaned ownership.
  */
@@ -1369,6 +1387,14 @@ static NSImage  *fileImage = nil;
   return [self fileName];
 }
 
+/**
+ * Create a subclass of the currently selected class in the classes view.
+ */
+- (id) createSubclass: (id)sender
+{
+  return [classesView createSubclass: sender];
+}
+
 /**
  * Add an outlet/action to the classes view.
  */
@@ -1378,13 +1404,20 @@ static NSImage  *fileImage = nil;
   return self;
 }
 
+/**
+ * Create an instance of a given class.
+ */
+- (id) instantiateClass: (id)sender
+{
+  return [classesView instantiateClass: sender];
+}
+
 /**
  * Remove a class from the classes view
  */
 - (id) remove: (id)sender
 {
-  [classesView deleteSelection];
-  return self;
+  return [classesView removeClass: sender];
 }
 
 /**
@@ -1392,47 +1425,7 @@ static NSImage  *fileImage = nil;
  */
 - (id) loadClass: (id)sender
 {
-  NSArray	*fileTypes = [NSArray arrayWithObjects: @"h", @"H", nil];
-  NSOpenPanel	*oPanel = [NSOpenPanel openPanel];
-  int		result;
-
-  [oPanel setAllowsMultipleSelection: NO];
-  [oPanel setCanChooseFiles: YES];
-  [oPanel setCanChooseDirectories: NO];
-  result = [oPanel runModalForDirectory: nil
-				   file: nil
-				  types: fileTypes];
-  if (result == NSOKButton)
-    {
-      NSString *filename = [oPanel filename];
-
-      NS_DURING
-	{
-	  if(![classManager parseHeader: filename])
-	    {
-	      NSString *file = [filename lastPathComponent];
-	      NSString *message = [NSString stringWithFormat: 
-					      _(@"Unable to parse class in %@"),file];
-	      NSRunAlertPanel(_(@"Problem parsing class"), 
-			      message,
-			      nil, nil, nil);
-	    }
-	  else
-	    {
-	      return self;
-	    }
-	}
-      NS_HANDLER
-	{
-	  NSString *message = [localException reason];
-	  NSRunAlertPanel(_(@"Problem parsing class"), 
-			  message,
-			  nil, nil, nil);
-	}
-      NS_ENDHANDLER
-    }
-
-  return nil;
+  return [classesView loadClass: sender];
 }
 
 /**
@@ -1440,55 +1433,7 @@ static NSImage  *fileImage = nil;
  */
 - (id) createClassFiles: (id)sender
 {
-  NSSavePanel		*sp;
-  NSString              *className = [classesView selectedClassName];
-  int			result;
-
-  sp = [NSSavePanel savePanel];
-  [sp setRequiredFileType: @"m"];
-  [sp setTitle: _(@"Save source file as...")];
-  if ([self fileName] == nil)
-    {
-      result = [sp runModalForDirectory: NSHomeDirectory() 
-		   file: [className stringByAppendingPathExtension: @"m"]];
-    }
-  else
-    {
-      result = [sp runModalForDirectory: 
-		     [[self fileName] stringByDeletingLastPathComponent]
-		   file: [className stringByAppendingPathExtension: @"m"]];
-    }
-
-  if (result == NSOKButton)
-    {
-      NSString *sourceName = [sp filename];
-      NSString *headerName;
-
-      [sp setRequiredFileType: @"h"];
-      [sp setTitle: _(@"Save header file as...")];
-      result = [sp runModalForDirectory: 
-		     [sourceName stringByDeletingLastPathComponent]
-		   file: 
-		     [[[sourceName lastPathComponent]
-			stringByDeletingPathExtension] 
-		       stringByAppendingString: @".h"]];
-      if (result == NSOKButton)
-	{
-	  headerName = [sp filename];
-	  NSDebugLog(@"Saving %@", className);
-	  if (![classManager makeSourceAndHeaderFilesForClass: className
-			     withName: sourceName
-			     and: headerName])
-	    {
-	      NSRunAlertPanel(_(@"Alert"), 
-			      _(@"Could not create the class's file"),
-			      nil, nil, nil);
-	    }
-	  
-	  return self;
-	}
-    }
-  return nil;
+  return [classesView createClassFiles: sender];
 }
 
 /**
@@ -1653,12 +1598,35 @@ static NSImage  *fileImage = nil;
   [editors removeAllObjects];
 }
 
+static void _real_close(GormDocument *self,
+			NSEnumerator *enumerator)
+{
+  id                obj;
+  NSNotificationCenter *nc = [NSNotificationCenter defaultCenter];
+
+  while ((obj = [enumerator nextObject]) != nil)
+    {
+      if ([obj isKindOfClass: [NSWindow class]])
+        {
+          [obj setReleasedWhenClosed: YES];
+          [obj close];
+        }
+    }
+
+  // deactivate the document...
+  [self setDocumentActive: NO];
+  [self closeAllEditors]; // shut down all of the editors..
+  [nc postNotificationName: IBWillCloseDocumentNotification object: self];
+  [nc removeObserver: self]; // stop listening to all notifications.
+}
+
 /**
  * Close the document and all windows associated.  Mark this document as closed.
  */
 - (void) close
 {
-  isDocumentOpen = NO;
+  isDocumentOpen = NO; 
+  _real_close(self, [nameTable objectEnumerator]);
   [super close];
 }
 
@@ -1670,28 +1638,10 @@ static NSImage  *fileImage = nil;
 - (void) handleNotification: (NSNotification*)aNotification
 {
   NSString *name = [aNotification name];
-  NSNotificationCenter	*nc = [NSNotificationCenter defaultCenter];
 
   if ([name isEqual: NSWindowWillCloseNotification] && isDocumentOpen)
     {
-      NSEnumerator	*enumerator;
-      id		obj;
-      
-      enumerator = [nameTable objectEnumerator];
-      while ((obj = [enumerator nextObject]) != nil)
-	{
-	  if ([obj isKindOfClass: [NSWindow class]])
-	    {
-	      [obj setReleasedWhenClosed: YES];
-	      [obj close];
-	    }
-	}
-
-      // deactivate the document...
-      [self setDocumentActive: NO];
-      [self closeAllEditors]; // shut down all of the editors..
-      [nc postNotificationName: IBWillCloseDocumentNotification object: self];
-      [nc removeObserver: self]; // stop listening to all notifications.
+      _real_close(self, [nameTable objectEnumerator]);
       isDocumentOpen = NO;
     }
   else if ([name isEqual: NSWindowDidBecomeKeyNotification] && isDocumentOpen)
@@ -1806,81 +1756,12 @@ static NSImage  *fileImage = nil;
 	  Class cls = [aNotification object];
 	  id mgr = [(IBResourceManager *)[cls alloc] initWithDocument: self];
 	  [resourceManagers addObject: mgr];
+  	  [IBResourceManager registerForAllPboardTypes:window
+	  			inDocument:self];
 	}
     }
 }
 
-/**
- * Create an instance of a given class.
- */
-- (id) instantiateClass: (id)sender
-{
-  NSString *object = [classesView selectedClassName];
-  GSNibItem *item = nil;
-  
-  if([object isEqualToString: @"FirstResponder"])
-    return nil;
-  
-  if([classManager isSuperclass: @"NSView" linkedToClass: object] ||
-     [object isEqual: @"NSView"])
-    {
-      Class cls;
-      NSString *className = object;
-      BOOL isCustom = [classManager isCustomClass: object];
-      id instance;
-      
-      if(isCustom)
-	{
-	  className = [classManager nonCustomSuperClassOf: object];
-	}
-      
-      // instantiate the object or it's substitute...
-      cls = NSClassFromString(className);
-      if([cls respondsToSelector: @selector(allocSubstitute)])
-	{
-	  instance = [cls allocSubstitute];
-	}
-      else
-	{
-	  instance = [cls alloc];
-	}
-      
-      // give it some initial dimensions...
-      if([instance respondsToSelector: @selector(initWithFrame:)])
-	{
-	  instance = [instance initWithFrame: NSMakeRect(10,10,380,280)];
-	}
-      else
-	{
-	  instance = [instance init];
-	}
-      
-      // add it to the top level objects...
-      [self setName: nil forObject: instance];
-      [self attachObject: instance toParent: nil];
-      
-      // we want to record if it's custom or not and act appropriately...
-      if(isCustom)
-	{
-	  NSString *name = [self nameForObject: instance];
-	  [classManager setCustomClass: object
-			forName: name];
-	}
-
-      [self changeToViewWithTag: 0];
-      NSLog(@"Instantiate NSView subclass %@",object);	      
-    }
-  else
-    {
-      item = [[GormObjectProxy alloc] initWithClassName: object];
-      [self setName: nil forObject: item];
-      [self attachObject: item toParent: nil];      
-      [self changeToViewWithTag: 0];
-    }
-  
-  return self;
-}
-
 /**
  * Returns YES, if document is active.
  */
@@ -2729,6 +2610,57 @@ static NSImage  *fileImage = nil;
   return removed;
 }
 
+/**
+ * Refresh all connections to any and all instances of className.  Checks if
+ * the class has the action/outlet present and deletes it, if it doesn't.
+ */
+- (void) refreshConnectionsForClassNamed: (NSString *)className
+{
+  NSEnumerator *en = [connections objectEnumerator];
+  NSMutableArray *removedConnections = [NSMutableArray array];
+  id<IBConnectors> c = nil;
+  
+  // first find all of the connections...
+  while ((c = [en nextObject]) != nil)
+    {
+      NSString *srcClass = [[c source] className];
+      NSString *dstClass = [[c destination] className];
+      NSString *label = [c label];
+      
+      if ([srcClass isEqualToString: className] ||
+	  [classManager isSuperclass: className 
+			linkedToClass: srcClass])
+	{
+	  if([c isKindOfClass: [NSNibOutletConnector class]])
+	    {
+	      if([classManager outletExists: label onClassNamed: className] == NO)
+		{
+		  [removedConnections addObject: c];
+		}
+	    }	      
+	}
+      else if([dstClass isEqualToString: className] ||
+	      [classManager isSuperclass: className 
+			    linkedToClass: dstClass])
+	{
+	  if([c isKindOfClass: [NSNibControlConnector class]])
+	    {
+	      if([classManager actionExists: label onClassNamed: className] == NO)
+		{
+		  [removedConnections addObject: c];
+		}
+	    }
+	}
+    }
+  
+  // then remove them.
+  en = [removedConnections objectEnumerator];
+  while((c = [en nextObject]) != nil)
+    {
+      [self removeConnector: c];
+    }
+}
+
 /**
  * Rename connections connected to an instance of on class to another.
  */
@@ -3575,6 +3507,11 @@ static NSImage  *fileImage = nil;
   [savedEditors removeAllObjects];
 }
 
+- (void) setFileType: (NSString *)type
+{
+  [super setFileType: type];
+  [filePrefsManager setFileTypeName: type];
+}
 @end
 
 @implementation GormDocument (MenuValidation)
@@ -3682,3 +3619,4 @@ willBeInsertedIntoToolbar: (BOOL)flag
 		  nil];
 }
 @end
+
