@@ -15,7 +15,7 @@
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
-   version 2 of the License, or (at your option) any later version.
+   version 3 of the License, or (at your option) any later version.
    
    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -85,7 +85,7 @@ NSString *formatVersion(int version)
 
 + (int) currentVersion
 {
-  return appVersion(1,1,0); 
+  return appVersion(1,2,2); 
 }
 
 - (void) awakeFromNib
@@ -296,6 +296,16 @@ NSString *formatVersion(int version)
   return ([targetVersionName isEqual: @"Latest Version"]);
 }
 
+- (void) setFileTypeName: (NSString *)ft
+{
+  [fileType setStringValue: ft];
+}
+
+- (NSString *) fileTypeName
+{
+  return [fileType stringValue];
+}
+
 // Data Source
 - (int) numberOfRowsInTableView: (NSTableView *)aTableView
 {
