@@ -11,7 +11,7 @@
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -83,6 +83,13 @@
 }
 @end
 
+@implementation NSView (GormObjectAdditions)
+- (NSString*) helpInspectorClassName
+{
+  return @"GormHelpInspector";
+}
+@end
+
 @implementation	GormObjectEditor
 
 static NSMapTable	*docMap = 0;
@@ -220,38 +227,15 @@ static NSMapTable	*docMap = 0;
 - (unsigned) draggingEntered: (id<NSDraggingInfo>)sender
 {
   NSArray   *pbTypes = nil;
-  NSString  *type = nil;
-  NSArray   *mgrTypes = nil; 
   
   // Get the resource manager first, if nil don't bother calling the rest...
   dragPb = [sender draggingPasteboard];
   pbTypes = [dragPb types];
-  resourceManager = [(GormDocument *)document resourceManagerForPasteboard: dragPb];
-
-  if(resourceManager != nil)
-    {
-      mgrTypes = [resourceManager resourcePasteboardTypes];
-      type = [mgrTypes firstObjectCommonWithArray: pbTypes];
-    }
-
-  if (type != nil)
-    {
-      dragType = type;
-    }
-  else if ([pbTypes containsObject: GormLinkPboardType] == YES)
+  
+  if ([pbTypes containsObject: GormLinkPboardType] == YES)
     {
       dragType = GormLinkPboardType;
     }
-  else if ([pbTypes containsObject: NSFilenamesPboardType] == YES)
-    {
-      NSArray *data = [dragPb propertyListForType: NSFilenamesPboardType];
-      NSString *fileName = [data objectAtIndex: 0];
-      NSString *ext = [fileName pathExtension];
-
-      [(GormDocument *)document changeToTopLevelEditorAcceptingTypes: pbTypes
-		       andFileType: ext]; 
-      dragType = nil;
-    }
   else
     {
       dragType = nil;
@@ -262,11 +246,7 @@ static NSMapTable	*docMap = 0;
 
 - (unsigned) draggingUpdated: (id<NSDraggingInfo>)sender
 {
-  if ([[resourceManager resourcePasteboardTypes] containsObject: dragType]) 
-    {
-      return NSDragOperationCopy;
-    }
-  else if (dragType == GormLinkPboardType)
+  if (dragType == GormLinkPboardType)
     {
       NSPoint	loc = [sender draggingLocation];
       int	r, c;
@@ -315,11 +295,6 @@ static NSMapTable	*docMap = 0;
       NSDebugLog(@"Recieved notification");
       [self setCellSize: defaultCellSize()];
     }
-  else if([name isEqual: IBResourceManagerRegistryDidChangeNotification])
-    {
-      [IBResourceManager registerForAllPboardTypes: self
-			 inDocument: document];
-    }
 }
 
 /*
@@ -343,10 +318,8 @@ static NSMapTable	*docMap = 0;
       NSButtonCell	*proto;
 
       document = aDocument;
-
-      [IBResourceManager registerForAllPboardTypes: self
-			 inDocument: document];
-
+      
+      [self registerForDraggedTypes:[NSArray arrayWithObject:GormLinkPboardType]];
       [self setAutosizesCells: NO];
       [self setCellSize: defaultCellSize()];
       [self setIntercellSpacing: NSMakeSize(8,8)];
@@ -445,6 +418,7 @@ static NSMapTable	*docMap = 0;
 		     owner: self];
 	  [pb setString: name forType: GormLinkPboardType];
 	  [NSApp displayConnectionBetween: obj and: nil];
+	  [NSApp startConnecting];
 
 	  [self dragImage: [NSApp linkImage]
 		       at: loc
@@ -463,12 +437,7 @@ static NSMapTable	*docMap = 0;
 
 - (BOOL) performDragOperation: (id<NSDraggingInfo>)sender
 {
-  if ([[resourceManager resourcePasteboardTypes] containsObject: dragType]) 
-    {
-      [resourceManager addResourcesFromPasteboard: dragPb];
-      return YES;
-    }
-  else if (dragType == GormLinkPboardType)
+  if (dragType == GormLinkPboardType)
     {
       NSPoint	loc = [sender draggingLocation];
       int	r, c;
@@ -505,14 +474,7 @@ static NSMapTable	*docMap = 0;
   /*
    * Tell the source that we will accept the drop if we can.
    */
-  if ([[resourceManager resourcePasteboardTypes] containsObject: dragType]) 
-    {
-      /*
-       * We can accept objects dropped anywhere.
-       */
-      return YES;
-    }
-  else if (dragType == GormLinkPboardType)
+  if (dragType == GormLinkPboardType)
     {
       NSPoint	loc = [sender draggingLocation];
       int	r, c;
@@ -563,6 +525,15 @@ static NSMapTable	*docMap = 0;
       [mgr setClassInspector];
     }
 }
+
+- (void) addObject:(id)anObject
+{
+  [super addObject:anObject];
+  /* we need to do this for palettes which can drop top level objects */
+  [(GormDocument *)document changeToViewWithTag:0];
+}
+
+
 @end
 
 
