@@ -9,7 +9,7 @@
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  * 
  * This program is distributed in the hope that it will be useful,
@@ -122,8 +122,6 @@
 
 static BOOL currently_displaying = NO;
 
-
-
 @implementation	GormViewEditor
 
 - (void) encodeWithCoder: (NSCoder*)aCoder
@@ -1185,6 +1183,7 @@ static BOOL currently_displaying = NO;
 	  owner: self];
       [pb setString: name forType: GormLinkPboardType];
       [NSApp displayConnectionBetween: anObject and: nil];
+      [NSApp startConnecting];
       
       [self dragImage: [NSApp linkImage]
 	    at: dragPoint
@@ -1418,13 +1417,13 @@ static BOOL currently_displaying = NO;
 	    withObject: self];
 }
 
-- (void) displayIfNeededInRectIgnoringOpacity: (NSRect) rect
+- (void) drawRect: (NSRect) rect
 {
   if (currently_displaying == NO)
     {
       [[self window] disableFlushWindow];
       currently_displaying = YES;
-      [super displayIfNeededInRectIgnoringOpacity: rect];
+      [super drawRect: rect];
       [self lockFocus];
       [self postDraw: rect];
       [self unlockFocus];
@@ -1434,36 +1433,13 @@ static BOOL currently_displaying = NO;
     }
   else
     {
-      [super displayIfNeededInRectIgnoringOpacity: rect];
+      [super drawRect: rect];
       [self lockFocus];
       [self postDraw: rect];
       [self unlockFocus];
     }    
 }
 
-- (void) displayRectIgnoringOpacity: (NSRect) rect
-{
-  if (currently_displaying == NO)
-    {
-      [[self window] disableFlushWindow];
-      currently_displaying = YES;
-      [super displayIfNeededInRectIgnoringOpacity: rect];
-      [self lockFocus];
-      [self postDraw: rect];
-      [self unlockFocus];
-      [[self window] enableFlushWindow];
-      [[self window] flushWindow];
-      currently_displaying = NO;
-    }
-  else
-    {
-      [super displayIfNeededInRectIgnoringOpacity: rect];
-      [self lockFocus];
-      [self postDraw: rect];
-      [self unlockFocus];
-    }
-}
-
 - (BOOL) acceptsTypeFromArray: (NSArray*)types
 {
   return NO;
@@ -1537,121 +1513,6 @@ static BOOL currently_displaying = NO;
 
 
 @implementation GormViewEditor (ResponderAdditions)
-
-- (void) keyDown: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder keyDown: theEvent];
-  else
-    return [self noResponderFor: @selector(keyDown:)];
-}
-
-- (void) keyUp: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder keyUp: theEvent];
-  else
-    return [self noResponderFor: @selector(keyUp:)];
-}
-
-- (void) otherMouseDown: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder otherMouseDown: theEvent];
-  else
-    return [self noResponderFor: @selector(otherMouseDown:)];
-}
-
-- (void) otherMouseDragged: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder otherMouseDragged: theEvent];
-  else
-    return [self noResponderFor: @selector(otherMouseDragged:)];
-}
-
-- (void) otherMouseUp: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder otherMouseUp: theEvent];
-  else
-    return [self noResponderFor: @selector(otherMouseUp:)];
-}
-
-
-- (void) mouseDragged: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder mouseDragged: theEvent];
-  else
-    return [self noResponderFor: @selector(mouseDragged:)];
-}
-
-- (void) mouseEntered: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder mouseEntered: theEvent];
-  else
-    return [self noResponderFor: @selector(mouseEntered:)];
-}
-
-- (void) mouseExited: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder mouseExited: theEvent];
-  else
-    return [self noResponderFor: @selector(mouseExited:)];
-}
-
-- (void) mouseMoved: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder mouseMoved: theEvent];
-  else
-    return [self noResponderFor: @selector(mouseMoved:)];
-}
-
-- (void) mouseUp: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder mouseUp: theEvent];
-  else
-    return [self noResponderFor: @selector(mouseUp:)];
-}
-
-- (void) rightMouseDown: (NSEvent*)theEvent
-{
-  if (_next_responder != nil)
-    return [_next_responder rightMouseDown: theEvent];
-  else
-    return [self noResponderFor: @selector(rightMouseDown:)];
-}
-
-- (void) rightMouseDragged: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder rightMouseDragged: theEvent];
-  else
-    return [self noResponderFor: @selector(rightMouseDragged:)];
-}
-
-- (void) rightMouseUp: (NSEvent*)theEvent
-{
-  if (_next_responder)
-    return [_next_responder rightMouseUp: theEvent];
-  else
-    return [self noResponderFor: @selector(rightMouseUp:)];
-}
-
-- (void) scrollWheel: (NSEvent *)theEvent
-{
-  if (_next_responder)
-    return [_next_responder scrollWheel: theEvent];
-  else
-    return [self noResponderFor: @selector(scrollWheel:)];
-}
-
-
 - (BOOL) acceptsFirstMouse: (NSEvent*)theEvent
 {
   return YES;
