@@ -10,7 +10,7 @@
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
+   the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
@@ -36,13 +36,18 @@
 
 #include <InterfaceBuilder/IBInspector.h>
 
-@class NSTextField;
+@class NSTextField, NSPopUpButton;
 
 @interface GormMenuItemAttributesInspector : IBInspector
 {
   NSTextField	*titleText;
   NSTextField	*shortCut;
   NSTextField	*tagText;
+  NSPopUpButton *keyPopup;
+  id             altBtn;
+  id             ctrlBtn;
+  id             shiftBtn;
+  id             cmdBtn;
 }
 @end
 
